--[[
Name: AlarLib-2.0
Revision: $Rev: 52286 $
Author: Alar (alar@aspide.it)
Website: http://wiki.wowace.com/index.php/AlarLib-2.0
Documentation: http://wiki.wowace.com/index.php/AlarLib-2.0
SVN: $HeadUrl:$
Description: Generic library
Dependencies: Ace2, AlarClass-2.0,Babble-Inventory-2.2
License: LGPL v2.1
--]]
--DEFAULT_CHAT_FRAME:AddMessage(GetTime().. " Alarlib-2.0",1,0,0)

local MAJOR_VERSION = "AlarLib-2.0"
local MINOR_VERSION = tonumber(string.sub("$Revision: 52286 $", 12, -3))
local function pp(arg)
    --arg="pp" .. MINOR_VERSION .. ":"  .. tostring(arg)
    --DEFAULT_CHAT_FRAME:AddMessage(arg,1,0.5,0.2)
end
if (not IsAddOnLoaded('Ace2')) then
    local nome,title=GetAddOnInfo('Ace2')
    if (title) then
        EnableAddOn(nome)
        error(MAJOR_VERSION .. " requires Ace2. I enabled it for you. Please ReloadUI or Relog")
    else
        error(MAJOR_VERSION .. " requires Ace2. Please install it.")
    end
end
-- This ensures the code is only executed if the library doesn't already exist, or is a newer version
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then
    pp("Skipping AlarLib revision " .. MAJOR_VERSION .. '.' .. MINOR_VERSION)
    return
end
function CheckLibraries(libs)
    for desc,lib in ipairs(libs) do
        lib=lib or desc
        if not AceLibrary:HasInstance(lib) then
            DEFAULT_CHAT_FRAME:AddMessage(desc .. "|r",1,0.5,0.2)
            local name,title=GetAddOnInfo(desc)
            if (title) then  
                EnableAddOn(name)
                if (IsAddOnLoadOnDemand(name)) then
                    LoadAddOn(name)
                else
                    DEFAULT_CHAT_FRAME:AddMessage("Enabling it.You need to relog)",1,0.5,0.2)
                end
            else
                DEFAULT_CHAT_FRAME:AddMessage("Please install it",1,0.5,0.2)
            end
        end
    end
end
local new, del
do
	local list = setmetatable({}, {__mode="k"})
	function new()
		local t = next(list)
		if t then
			list[t] = nil
			return t
		else
			return {}
		end
	end
	function del(t)
		setmetatable(t, nil)
		for k in pairs(t) do
			t[k] = nil
		end
		list[t] = true
	end
end
local libs={
['Ace2']='AceOO-2.0',
['Babble-Inventory-2.2']=nil,
DewdropLib='DewDrop-2.0',
TabletLib='Tablet-2.0',
['Waterfall-1.0']=nil,
}
CheckLibraries(libs)
local lib=new()

-----------------------------------------------------
-- Executed only for the newest version
-- Backdrops definition
local Backdrops={
dialog = {
    bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
    edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border",
    tile=true,
    tileSize=32,
    edgeSize=32,
    insets={bottom=5,left=5,right=5,top=5}
},
party = {
    bgFile="Interface\\CharacterFrame\\UI-Party-Background",
    edgeFile="Interface\\CharacterFrame\\UI-Party-Border",
    tile=true,
    tileSize=32,
    edgeSize=32,
    insets={bottom=32,left=32,right=32,top=32}
},
tooltip = {
    bgFile="Interface\\Tooltips\\UI-Tooltip-Background",
    edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
    tile=true,
    tileSize=16,
    edgeSize=16,
    insets={bottom=5,left=5,right=5,top=5}
},
border = {
    bgFile="",
    edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
    tile=true,
    tileSize=16,
    edgeSize=16,
    insets={bottom=5,left=5,right=5,top=5}
},
tutorial = {
    bgFile="Interface\\TutorialFrame\\TutorialFrameBackground",
    edgeFile="Interface\\Tooltips\\UI-Tooltip-Border",
    tile=true,
    tileSize=32,
    edgeSize=32,
    insets={bottom=7,left=7,right=7,top=7}
}
}
Backdrops.default=Backdrops.tutorial

----- New Global Function, I check if someone already defined then
if (not string.capitalize) then
    function string.capitalize(stringa)
        return string.gsub(strlower(stringa),
                "%w",function (s) return strupper(s) end,1)
    end
end
if (not table.kpairs) then
    function table.kpairs(t,f)
      local a = {}
      for n in pairs(t) do table.insert(a, n) end
      table.sort(a, f)
      local i = 0      -- iterator variable
      local iter = function ()   -- iterator function
        i = i + 1
        if a[i] == nil then return nil
        else
            local k=a[i]
            a[i]=nil -- Should optimize memory usage
            return k, t[k]
        end
      end
      return iter
    end
end
if (not kpairs) then
    kpairs=table.kpairs
end
-------------- Constants area
--- WOW or PSEUDO WOW
UIERRORS_HOLD_TIME=UIERRORS_HOLD_TIME or 1.0 -- I think this is no longer used
MAX_BAGS=MAX_BAGS or 4
-- General COlors
AMO_AZURE                   ="|c000C92DC"
AMO_Azure                   ={r=0.05,g=0.57,b=0.86}
AMO_BLACK                   ="|c00000000";
AMO_BLUE                    ="|c000000FF";
AMO_BLUE                    ="|c000000ff";
AMO_BRIGHTGREY              ="|c00D0D0D0";
AMO_CONNECTION_COLOR        ="|c0033FF66";
AMO_COPPER                  ="|c00CC9900";
AMO_DEBUG_COLOR             ="|c0000FF00";
AMO_GOLD                    ="|c00FFFF66";
AMO_GRAY                    ="|c00808080";
AMO_Gray                    ={r=0.5,g=0.5,b=0.5}
AMO_GREEN                   ="|c0020ff20";
AMO_Green                   ={r=0.12,g=1,b=0.12}
AMO_GREEN2                  ="|c0000C000";
AMO_Green2                   ={r=0.0,g=0.77,b=0.0}
AMO_GREY                    ="|c00909090";
AMO_GUILDCHAT               ="|c00269926"
AMO_HIGHLIGHT_COLOR_CODE    ="|cffffffff";
AMO_LIGHTBLUE               ="|cff515179"
AMO_LIGHTYELLOW             ="|cffffff9a";
AMO_MONEY_COLOR             ="|c00FFCC33";
AMO_ORANGE                  ="|c00ff9900";
AMO_Orange                  ={r=1.0,g=0.6,b=0.0}
AMO_PARTYCHAT               ="|c00515179"
AMO_PURPLE                  ="|c00800080"
AMO_Purple                  ={r=0.50,g=0.00,b=0.50}
AMO_RAIDCHAT                ="|c0066331a"
AMO_RED                     ="|c00ff2020";
AMO_Red                     ={r=0.96,g=0.10,b=0.0}
AMO_RED2                    ="|c00F41400";
AMO_SILVER                  ="|c00C0C0C0";
AMO_STATUS_COLOR            ="|c000066FF";
AMO_WHITE                   ="|c00FFFFFF";
AMO_White                   ={r=1,g=1,b=1}
AMO_YELLOW                  ="|cffffd200";
AMO_Yellow                  ={r=1.0,g=0.93,b=0.10}
AMO_YELLOW2                 ="|c00FFED1A";
AMO_ENDCOLOR                ="|r"
-- Class colors
AMO_COLOR_CLASS_DRUID       ="|C00ff7d0a";
AMO_COLOR_CLASS_Druid       ={r=1.0,g=0.49,b=0.04}
AMO_COLOR_CLASS_HUNTER      ="|C00abd473";
AMO_COLOR_CLASS_Hunter      ={r=0.67,g=0.83,b=0.45}
AMO_COLOR_CLASS_MAGE        ="|C0069ccf0";
AMO_COLOR_CLASS_Mage        ={r=0.41,0.8,0.94};
AMO_COLOR_CLASS_PALADIN     ="|C00f58cba";
AMO_COLOR_CLASS_Paladin     ={r=0.96,g=0.55,0.73};
AMO_COLOR_CLASS_PRIEST      ="|C00ffffff";
AMO_COLOR_CLASS_Priest      ={r=1.0,g=1.0,b=1.0};
AMO_COLOR_CLASS_ROGUE       ="|C00fff569";
AMO_COLOR_CLASS_Rogue       ={r=1.0,g=0.96,b=0.41};
AMO_COLOR_CLASS_SHAMAN      ="|C00f58cba";
AMO_COLOR_CLASS_Shaman      ={r=0.96,g=0.55,0.73};
AMO_COLOR_CLASS_WARLOCK     ="|C009482ca";
AMO_COLOR_CLASS_Warlock     ={r=0.58,g=0.51,b=0.79}
AMO_COLOR_CLASS_WARRIOR     ="|C00c79c6e";
AMO_COLOR_CLASS_Warrior     ={r=0.78,g=0.61,b=0.43}
function AMO_color(colore)
    if (colore) then
        return colore.r,colore.g,colore.b
    else
        return 1,1,1
    end
end
BINDING_HEADER_AMO_HEADER="Alar General Bindings"
--Buff Texture
AMO_BUFFID=AMO_BUFFID or {}
AMO_BUFFID["Soulstone Resurrection"]="Spell_Shadow_SoulGem"
---- Old generic function. Compatibility issuese
local BI=AceLibrary("Babble-Inventory-2.2")
function AMO_TRACK(...)
    AMO:Print(...)
end
function AMO_TTAdd(frame,message)
    AMO:TTAdd(frame,message)
end
function AMO_TTClose()
    AMO:TTClose()
end

function AMO_TTFade()
    AMO:TTFade()
end

function AMO_TTFormat(lmessage,rmessage)
    AMO:TTOpen(lmessage,rmessage)
end
function AMO_TTOpen(message,message2)
    AMO:TTOpen(message)
end

--------------------------------------------------------------------------------
local AMO_Events={
    ADDON_ACTION_BLOCKED=nil,
    ADDON_ACTION_FORBIDDEN=nil,
    CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS=nil,
    MACRO_ACTION_BLOCKED=nil,
    MACRO_ACTION_FORBIDDEN=nil,
    ACTIONBAR_SLOT_CHANGED=0,
    CHAT_MSG_ADDON=0, -- Filtered out
    PLAYER_LEVEL_UP=0,
    PLAYER_REGEN_DISABLED=0,-- Time to make last precombat changes
    PLAYER_REGEN_ENABLED=0, -- TIme to make first postcombat changes
    SPELLS_CHANGED=0,
    UPDATE_BINDINGS=0,
    ZONE_CHANGED_NEW_AREA=0,
    UNIT_SPELLCAST_SENT=nil,
    UNIT_SPELLCAST_SUCCEEDED=nil,
}
lib=AceLibrary("AceAddon-2.0"):new("AceEvent-2.0","AceConsole-2.0","AceDebug-2.0")
--local AMO={}
lib.ID="AMO"
local quiver=BI["Quiver"]
local ammopouch=BI["Ammo Pouch"]
BI=nil
function lib:CheckLibraries(libs)
    CheckLibraries(libs)
end
function lib:FindQuiver()
    return self:FindBag(quiver)
end
function lib:FindAmmoPouch()
    return self:FindBag(ammopouch)
end
function lib:Tooltip2Table(tooltip)
    local name=tooltip:GetName()
    local tt={}
    for i=1,tooltip:NumLines() do
       local mytext = getglobal(name .. "TextLeft" .. i)
       table.insert(tt,mytext:GetText())
    end
    return tt
end
function lib:FindBag(key)
    if (not key) then
        return 0
    end
    local bag
    for i=1,MAX_BAGS do
        bag=GetBagName(i)
        if (bag) then
            local slot=ContainerIDToInventoryID(i)
            local tip=self:GetTooltip()
            tip:SetInventoryItem("player",slot)
            local tt=self:Tooltip2Table(tip)
            if (self:TableFind(key,tt,true)>0) then
                return i
            end
        end
    end
    return 0
end

function lib:GetFreeSlotInBag(bag,slot,tipo)
    if (not bag) then
        return 0
    end
    slot=slot or 0
    if (slot==0) then
        slot=1
    end
    local l=GetContainerNumSlots(bag)
    for i=slot,l do
        local item=self:GetItemInfo(bag,i)
        if (item and tipo)  then
            if (item.Type ~= tipo) then
                return i
            end
        elseif (not item) then
            return i
        end
    end
    return  0
end
function lib:GetSmallestBag()
    local small={id=1,size=999,slot=1}
    for i=1,MAX_BAGS do
        local l = GetContainerNumSlots(i)
        if (l > 0 and l < small.size) then
            small.size=l
            small.id=i
        end
    end
    if (small.size < 999) then
        return small
    else
        return nil
    end
end
function lib:ExplodeBagId(id)
    local Bag=math.floor(id/1000);
    local Slot=id-(Bag*1000)
    return Bag,Slot
end
function lib:ImplodeBagId(bag,slot)
    return bag * 1000 + slot
end
function lib:BagUse(bagID,slot)
    local bag=bagID
    if (not slot) then
        bag=math.floor(bagID/1000);
        slot=bagID-(bag*1000)
    end
    UseContainerItem(bag,slot)
end
function lib:BagCount(bagID)
    local Bag=math.floor(bagID/1000);
    local Slot=bagID-(Bag*1000)
    _,count=GetContainerItemInfo(Bag,Slot)
    return count or 0
end
function lib:BagIdInBag(Inbag,bagID)
    local Bag=math.floor(bagID/1000);
    return (Inbag == Bag)
end
function lib:ItemCount(buff,partialmatch)
    local bagId=self:BagScan(buff,partialmatch)
    if (bagId) then
        return self:BagCount(bagId)
    else
        return 0
    end
end
function lib:ScanBags()
    for i = 0,MAX_BAGS do
        local l = GetContainerNumSlots(i)
        if (l > 0) then
            for k = 1,l do
                local item=self:GetItemInfo(i,k)
                self.cache[item.Name]=item.Count
            end
        end
    end
end
-- Finds an object in bags
-- return BagId,link,name,count,texture
function lib:FindEmptySlots(bagtoavoid)
    return self:FindEmptySlot(bagtoavoid,true)
end
function lib:FindEmptySlot(bagtoavoid,all)
    bagtoavoid=bagtoavoid or -1
    local freeslots={}
    for i = 0,MAX_BAGS do
        if (not (i == bagtoavoid)) then
            local l = GetContainerNumSlots(i)
            for k = 1,l do
                if (not GetContainerItemInfo(i,k)) then
                    if (all) then
                        table.insert(freeslots,{bag=i,slot=k})
                    else
                        return i,k
                    end
                end
            end
        end
    end
    if (all) then
        return freeslots
    else
        return nil
    end
end
function lib:BagScan(buff,partialmatch)
    local l,item=self:BagFind(buff,partialmatch)
    if (l) then
        return l,item.Link
    end
    return
end

function lib:BagFind(buff,partialmatch)
    buff=strupper(buff)
    for i = 0,MAX_BAGS do
        local l = GetContainerNumSlots(i)
        for k = 1,l do
            local item=self:GetItemInfo(i,k) or {}
            if (item.Name and item.Texture) then
                if (strupper(item.Texture)==buff) or
                    (strupper(item.Name) == buff) then
                    return i*1000+k, item
                end
                if (partialmatch) then
                    if (string.find(strupper(item.Texture),buff)) or
                        (string.find(strupper(item.Name),buff)) then
                        return i*1000+k, item
                    end
                end
            end
        end
    end
end

function lib:GetItemInfo(bag,slot)
    local item={}
    local itemlink=bag
    if slot then
        itemlink=GetContainerItemLink(bag,slot)
    end
    if (not itemlink) then
        return nil
    end
    if (slot) then
        item.Texture,
        item.Count,
        item.Locked,
        item.Quality,
        item.Readable=GetContainerItemInfo(bag,slot)
    end
    item.Name,
    item.Link,
    item.Rarity,
    item.Level,
    item.MinLevel,
    item.Type,
    item.SubType,
    item.StackCount,
    item.EquipLoc,
    item.Texture=GetItemInfo(itemlink)
    return item
end

function lib:PickInBags(buff)
    if (buff) then
        local bagid=self:FindInBags(strlower(buff),true)
        if (bagid) then
            local bagID,slot=self:ExplodeBagId(bagid)
            ToggleBag(bagID)
            PickupContainerItem(bagID, slot)
        end
    end
end

function lib:iCmdEcho(subcmd,param,fullcmd)
    self:Printf("%s %s (%s)",subcmd,param,fullcmd)
end


function lib:iCmdTrack(arg1)
    self:AddEvent(strupper(arg1))
end
function lib:iCmdRuler(arg)
    arg=arg or "border"
    if ( not self.db.profile.forms.AlarRuler) then
        self.db.profile.forms.AlarRuler={storage={Scale=1.0}}
    end
    local form=self:CreatePanel("AlarRuler",{noTip=1,noClose=1,noSave=1,noCancel=1,noResize=1})
    local s=500
    form:SetHeight(s)
    form:SetWidth(s)
    for i=0,s,50 do
        self:FrameAddLLabel(form,"xxxx" .. i,"x" .. i ,i,20)
        self:FrameAddLLabel(form,"yyyy".. i,"y-" .. i ,-20,i*-1)
    end
	self:SetBackdrop(form,arg)
end
function lib:iCmdBuffs()
    local u="target"
    if (not UnitName(u)) then
        u="player"
    end
    local b =self:ScanBuffDebuff(u)
    for nome,dati in pairs(b) do
        self:Printf("%s (%s): %s",nome,dati.rank,dati.name)
    end
end
function lib:iCmdDebuffs()
    local u="target"
    if (not UnitName(u)) then
        u="player"
    end
    local b =self:ScanBuffDebuff(u,true)
    for nome,dati in pairs(b) do
        self:Printf("%s (%s)[%s]: %s",nome,dati.rank,rank.type,dati.name)
    end
end
function lib:iCmdMonitor()
    self.monitor=not self.monitor
    self:PrintBoolean("Monitor",self.monitor)
end
function lib:iCmdDump(structure)
    local func,message=loadstring('lib:DumpTable(' .. structure .. ")")
    if (type(func)=="function") then
        func()
    else
        lib:Error(message)
    end
end

function lib:iCmdWhere()
    local x,y=GetPlayerMapPosition("player")
    self:Printf("You are in %s (%d,%d)",GetRealZoneText(),x*100,y*100)
end
function lib:iCmdMouse()
    self:Print(GetMouseFocus())
end
function lib:iCmdPoi()
    local f=getglobal("POI")
    if (not f) then
        f = CreateFrame("Frame","POI",UIParent)
        f:SetFrameStrata("BACKGROUND")
        f:SetWidth(256)  -- Set These to whatever height/width is needed
        f:SetHeight(256) -- for your Texture
        local t = f:CreateTexture(nil,"BACKGROUND")
        t:SetTexture("Interface\\Minimap\\POIIcons")
        t:SetAllPoints(f)
        f.texture = t
        f:SetPoint("CENTER",0,0)
    end
    if (f:IsShown()) then
        f:Hide()
    else
        f:Show()
    end
end

function lib:SafeConcat(separator,...)
    local i=select("#",...)
    local fmt="";
    if (i > 0) then
        fmt=fmt .. string.rep("%s"..separator,i)
    end
    return self:Format(fmt,...)
end
-- It actually dumps any kind of variable
function lib:DumpTable(table)
    if (type(table) == "string") then
        table=getglobal(table)
    end
    self:PrintLiteral(table)
end
lib.Dump = lib.DumpTable
function lib:Scale(form,def,force)
-- Gestisce sia il rescaling, che l'applicazione del BackDrop
    local f=self:IsFrame(form)
    if (not f) then return end
    if (not force) then
        def=f.storage.Scale or def
    end
    f:SetScale(def or 1.0)

end
function lib:Channel(msg,channel)
    channel=channel or "SAY"
    SendChatMessage(msg,channel)
end
function lib:Format(...)
    local a={}
    for i=1,select("#",...) do
        local v=select(i,...)
        table.insert(a,tostring(v))
    end
    rc,message = pcall (format,unpack(a))
    if (not rc) then
        self:Print(AMO_RED .. message)
        return ""
    else
        return message
    end
end

function lib:ChooseClick(click)
    button=0
	if (click == "LeftButton") then
        button=1
	elseif (click == "MiddleButton") then
        button=2
	elseif (click == "RightButton") then
        button=3
    end
    if (IsControlKeyDown()) then
        button=button+3
    end
    if (IsAltKeyDown()) then
        button=button+6
    end
    return button
end

function lib:Boolean(toggle)
    local stato
    if (toggle) then
        stato = AMO_GREEN .. "ON"
    else
        stato= AMO_RED .. "OFF"
    end
    return stato
end
function lib:PrintBoolean(msg,toggle)
    self:Printf("%s is %s",msg,self:Boolean(toggle))
end
function lib:Same()
    self:Print("Originale")
end
function lib:Debug(...)
    local addon=self.ID or "AMO"
    self.CustomDebug(addon,nil,nil,nil,nil,nil,nil,...)
end
function lib:Print(...)
    local addon=self.ID or "AMO"
    pp("AMOPRINT")
    self.CustomPrint(addon,nil,nil,nil,nil,nil,nil,...)
end
function lib:Error(...)
    local rc,message=pcall(error,self:Format(...),3)
    local addon=self.ID or "AMO"
    self.CustomPrint(AMO_RED .. addon ..AMO_ENDCOLOR,nil,nil,nil,nil,nil,nil,message)
end
function lib:Notify(...)
    local addon=self.ID or "AMO"
    self.CustomPrint(AMO_ORANGE .. addon .. AMO_ENDCOLOR ,nil,nil,nil,nil,nil,nil,...)
end

function lib:Printf(...)
    self:Print(...)
end
function lib:SetColor(frame,c)
    if (not c) then
         c={r=0.5,g=0.5,b=0.5}
    end
    frame:SetTextColor(c.r,c.g,c.b)
end

function lib:Long(msg)
	UIErrorsFrame:AddMessage(msg, AMO_Yellow.r,AMO_Yellow.g,AMO_Yellow.b, 1.0, UIERRORS_HOLD_TIME * 20);
end
function lib:Onscreen_Orange(msg)
-- Viola = 50%,0%,50%
	UIErrorsFrame:AddMessage(msg, AMO_Orange.r,AMO_Orange.g,AMO_Orange.b, 1.0, UIERRORS_HOLD_TIME * 2);
end
function lib:Onscreen_Purple(msg)
-- Viola = 50%,0%,50%
	UIErrorsFrame:AddMessage(msg, AMO_Purple.r,AMO_Purple.g,AMO_Purple.b, 1.0, UIERRORS_HOLD_TIME * 8);
end
function lib:Onscreen_Yellow(msg)
-- Giallo = 100%,93%,22%
	UIErrorsFrame:AddMessage(msg, AMO_Yellow.r,AMO_Yellow.g,AMO_Yellow.b, 1.0, UIERRORS_HOLD_TIME);
end
function lib:Onscreen_Azure(msg)
-- Viola = 50%,0%,50%
	UIErrorsFrame:AddMessage(msg, AMO_Azure.r,AMO_Azure.g,AMO_Azure.b, 1.0, UIERRORS_HOLD_TIME);
end
function lib:Onscreen_Red(msg)
-- Rosso = 96%,10%,0%
	UIErrorsFrame:AddMessage(msg, AMO_Red.r,AMO_Red.g,AMO_Red.b, 1.0, UIERRORS_HOLD_TIME);
end
function lib:Onscreen_Green(msg)
-- Verde = 0%,77%,0%
	UIErrorsFrame:AddMessage(msg, AMO_Green.r,AMO_Green.g,AMO_Green.b, 1.0, UIERRORS_HOLD_TIME);
end

-----------------------------------------------------------
-- function lib:Show(self,form)
--
-- :Show() a form, with scaling and backdropping
--
function lib:Show(form)
    local f=form
    if (type(form)=="string") then
        f=getglobal(form)
    else
        form=f:GetName()
    end
    self:Scale(form,1.0,nil)
    f:Show()
end
function lib:Flip(form,condizione)
    local f=form
    if (type(form)=="string") then
        f=getglobal(form)
    end
    if (type(f) ~= "table") then
        return
    end
    if (condizione) then
        f:Show()
    else
        f:Hide()
    end
end
    -- AMo Help System
function lib:ShowHelp(page)
    if (not self.HELP) then
        self.HELP=AceLibrary("Waterfall-1.0")
        self.PAGES={}
        for id,t in ipairs(ACH_HELP_PAGES) do
            self.PAGES[id]={strsplit("\n",t)}
        end
        local Opts={}
        for i,t in ipairs(ACH_HELP_TITLES) do
            Opts[i]= {
                id=i,
                text=t
            }
        end
        self.HELP:Register("ACH_HELP",
        "tree",Opts,
        "title","Alar Mods Help System",
        "children",
            function(id)
                if (id) then
                    if (id > 1) then
                        self.HELP:AddControl(
                        'type','button',
                        'text','Configure',
                        'justifyH','CENTER',
                        'execArg1',id,
                        'execFunc',
                            function(id)
                                local lib=getglobal(ACH_CONFIG[id])
                                rc,message= pcall(function() lib:CmdConfig() end)
                                if (rc) then
                                    self.HELP:Close("ACH_HELP")
                                else
                                    self:Print("Failed CmdConfig",id,ACH_CONFIG[id])
                                end
                            end
                        )
                    end
                    for i,t in pairs(self.PAGES[id]) do
                        self.HELP:AddControl('type','label',"width",600,"text",t,"noNewLine",false,"justifyH","LEFT")
                    end
                    return ACH_HELP_TITLES[id]
                else
                    return "Choose a module to display help for"
                end
            end
            )
    end
    if (type(page) ~= "number") then
        page=1
    end
    self.HELP:Open("ACH_HELP",2)
    --AlarHelpFrameShow(page)
    --AlarHelpFrame:Show()
end
-- class checks
function lib:Is(class,target)
    target=target or "player"
    if (target == "player") then
        if (not self.Myclass) then
            _,self.Myclass=UnitClass("player")
        end
        return self.Myclass==strupper(class)
    else
        local    rc,_,unitclass=pcall(UnitClass,target)
        if (rc) then
            return unitclass==strupper(class)
        else
            return false
        end
    end
end
function lib:GetUnitDistance(x,y,unit)
    unit=unit or "player"
    local from={}
    local to={}
    from.x,from.y=GetPlayerMapPosition(unit)
    to.x=x
    to.y=y
    return self:GetDistance(from,to) * 10000
end
function lib:GetDistance(a,b)
--------------
-- Calculates distance betweeb 2 points given as
-- a.x,a.y and b.x,b.y
	 local x=b.x - a.x
	 local y=b.y -a.y
	 local d=x*x + y* y
	 local rc,distance=pcall(math.sqrt,d)
	 if (rc) then
        return distance
    else
        return 99999
    end
end
function lib:NewMacro(...)
    free=self:CreateMacro(...)
    if (free) then
        PickupMacro(free)
    else
        PlaySound("igQuestLogAbandonQuest");
        self:Onscreen_Red("No more macro slots left")
    end
end
function lib:CreateMacro(nome,testo,icona,locale)
    silent=true
    if (not icona) then
        icona=1
    end
    if (not locale) then
        locale=1
    end
    local free=GetMacroIndexByName(nome)
    if (free) then
        DeleteMacro(free)
    end
    free=CreateMacro(nome,icona,testo,1,locale)
    if (not free) then
        return nil
    else
        return free
    end
end

---- miscellaneous checks
function lib:IsFriend(player)
    self:Print("isfriend")
    local i
    for i =1,GetNumFriends() do
        local name,_,_,_,_ =GetFriendInfo(i)
        if (name == player) then
            return true
        end
    end
    return false
end
function lib:StringWidth(stringa)
    if (type(stringa) ~= "string") then
        stringa=""
    end
    AMO_CORELabel:SetText(stringa)
    return AMO_CORELabel:GetStringWidth() + 10
end

function lib:FindMe()
	local totallandmarks = GetNumMapLandmarks()
	local IAM=""
	local ld=999999999
	local x,y=GetPlayerMapPosition("player")
	local a={x=x,y=y}
	for i = 1, totallandmarks, 1 do
		local name, description, typ, x, y = GetMapLandmarkInfo(i);
		local b={x=x,y=y}
		local d=lib:GetDistance(a,b)
		if (d < ld) then
            ld=d
            IAM=tostring(name) .. "(" .. tostring(description) .. ")"
        end
    end
    return IAM
end
function lib:AttachHelp(f,helptext)
    --UiFrameFadeIn(f,secondi,Alfada,Alfaa)
    helptext=helptext or ""
    helptext=tostring(helptext)
    if (type(f) == "string") then
        f=getglobal(f)
    end
    if (not f) then
        self:Print(tostring(f) .. " is not a global name")
        return
    end
    if (type(f) ~= "table") then
        self:Print(tostring(f) .. " is not a frame")
        return
    end
    local canoperate=f:IsObjectType("Frame")
    if (canoperate) then
        local b=self:FrameAddHelpButton(f)
        b:SetPoint("TOPLEFT",f,"TOPRIGHT",-5,17)
        b:Show()
        self:TTAdd(b,helptext,true)
    end
end
function lib:IsFrame(tf,verbose)
    local f
    if (type(tf) == "table") then
        f=tf
    else
        f=getglobal(tf)
    end
    if (not f) then
        return verbose and self:Print(tf , " is not a global name")
    end
    if (type(f) ~= "table") then
        return verbose and self:Print(tf .. " is not a table")
    end
    local canoperate=f.IsObjectType and f:IsObjectType("Frame")
    if (canoperate) then
        return f
    else
        return verbose and self:Print(tf .. " is not a frame")
    end
end
local OnDragStart,OnDragStop,OnDragInfoOn,OnDragInfoOff,savepanel,restorepanel

function restorepanel(f)
    if (not f) then return end
    if (f.storage and f.storage.Manage) then
        if (type(f.storage.Point)=="table") then
            rc,message=pcall(function()
                f:SetPoint(unpack(f.storage.Point))
                end)
        end
        if (not rc) then
            AMO:PrintLiteral(message,f.storage.Point)
        end
        if (f.storage.Width) then
            f:SetWidth(f.storage.Width)
        end
        if (f.storage.Height) then
            f:SetHeight(f.storage.Height)
        end
        f:SetScale(f.storage.Scale)
        f:SetBackdrop(Backdrops[f.storage.Backdrop])
    else
        f:SetPoint("CENTER")
    end
end
function savepanel(f)
    if (not f or not f.storage) then return end
    f.storage.Scale=f:GetScale()
    f.storage.Point={f:GetPoint()}
    f.storage.Width=f:GetWidth()
    f.storage.Height=f:GetWidth()
    f.storage.Manage=1
end

function OnDragStart(this,...)
    AMO:Print("AMO->OnDragStart",this,...)
    if (IsAltKeyDown()) then
        AMO_TTOpen("Sizing....")
        this:StartSizing("BOTTOMRIGHT")
    else
        AMO_TTOpen("Moving....")
        this:StartMoving()
    end
end
function OnDragInfoOn(old,this,button)
    if (IsAltKeyDown()) then
        AMO_TTOpen("Ready to resize")
    else
        AMO_TTOpen(AMO_GREEN .."Right-click and drag:" .. AMO_ENDCOLOR .. " move\n" .. AMO_GREEN .. "Alt-Right-Click and drag:".. AMO_ENDCOLOR .. "resize")
    end
    if (type(old) == "function") then old(this,button) end
end
function OnDragInfoOff(old,this,button)
    AMO_TTFade()
    if (type(old) == "function") then old(this,button) end
end
function OnDragStop(this,...)
    AMO:Print("AMO->OnDragStop",this,...)
    AMO_TTFade()
    this:StopMovingOrSizing();
    if (type(this.storage)=="table") then
        savepanel(this)
    end
    local f=this:GetParent()
    if (f) then
        if (f:GetName() ~= "UIParent") then
            this:ClearAllPoints()
            this:SetPoint("TOPLEFT",
                this:GetLeft()-f:GetLeft(),
                this:GetTop()-f:GetTop())
        end
    end
end
local OnDragStopParent
function OnDragStopParent(this,...)
    OnDragDStop(this:GetParent(),'parent')
end
local OnDragStartParent
function OnDragStartParent(this)
    OnDragDStart(this:GetParent(),'parent')
end
function lib:MakeParentMovable(tf)
    local f=self:IsFrame(tf,true)
    local p=f:GetParent()
    if (not p) then
        return
    end
    self:MakeMovable(p,true)
    f:SetScript("OnDragStart",OnDragStartParent)
    f:SetScript("OnDragStop",OnDragStopParent)
end
function lib:MakeMovable(tf,noinfo)
    local f=self:IsFrame(tf,true)
    if (f) then
        f.oldparent=f:GetParent()
        f:SetMovable(true)
        f:EnableMouse(true)
        if (f:GetFrameStrata() == "BACKGROUND") then
            f:SetFrameStrata("LOW")
        end
        f:SetMovable(true)
        f:SetResizable(true)
        f:SetClampedToScreen(true)
        f:RegisterForDrag("RightButton")
        frame=f:GetName()
        if (not self.db.profile.forms[frame]) then
            self.db.profile.forms[frame]={}
        end
        f.storage=self.db.profile.forms[f:GetName()]
        if (not noinfo) then
            local omd=f:GetScript("OnMouseDown")
            local omu=f:GetScript("OnMouseUp")
            f:SetScript("OnMouseDown",function(...) OnDragInfoOn(...) end)
            f:SetScript("OnMouseUp",function(...) OnDragInfoOff(...) end)
        end
        f:SetScript("OnDragStart",OnDragStart)
        f:SetScript("OnDragStop",OnDragStop)
    else
        self:Print("Sorry, ",tf," is not a frame object")
    end
end
function lib:Parse(msg)
    self:Print("Parse",msg)
    if (not msg) then
        return nil
    end
    local cmd,subcmd,param = strsplit(" ",msg,3)
    local _,fullarg=strsplit(" ",msg,2)
    return cmd,subcmd,param,fullarg
end
function lib:TableFind(key,table,approx)
    if (not key) then
        return 0
    end
    if (type(table) ~= "table") then
        return 0
    end
    for i,v in ipairs(table) do
        if (approx) then
            if (string.find(v,key)) then
                return i
            end
        else
            if v==key then
                return i
            end
        end
    end
    return 0
end
--- Templateless frame functions
local function AMO_ChangeBackdrop(this)
    lib:PrintLiteral(this)
end
function lib:PanelRestore(f)
    self:Debug("PanelRestore")
    restorepanel(self:IsFrame(f))
end
function lib:PanelSave(f)
    self:Debug("PanelSave")
    savepanel(self:IsFrame(f))
end
function lib:CreatePanel(nome,p,show)
    p=p or {}
    p.name=nome or "Panel" .. math.ceil(time())
    local f=self:Create("Panel",p.name,UIParent)
    self:SetBackdrop(f,p.backdrop)
    f:SetParent(UIParent)
    f:SetHeight(400)
    f:SetWidth(400)
    f:SetScale(0.8) -- My default scale
    self:PanelRestore(f) -- If it was never saved, Panel restore does nothing
    -- I DONT save panel now, because the creation process could be change it
    -- and I dont want to create false defaults
    -- X Button
    if (p.X or not p.noX) then
        local b,x,y=self:FrameAddXButton(f)
    end
    if (p.Close and not p.noClose ) then
        local b,x,y=self:FrameAddCloseButton(f)
        if (type(p.closefunc) == "function") then
            b:SetScript("OnClick",p.closefunc)
        end
    end
    if (p.Save or not p.noSave) then
        local b,x,y=self:FrameAddSaveButton(f)
        if (type(p.savefunc) == "function") then
            b:SetScript("OnClick",p.savefunc)
        end
    end
    if (p.Cancel or not p.noCancel) then
        local b,x,y=self:FrameAddCancelButton(f)
        if (type(p.cancelfunc) == "function") then
            b:SetScript("OnClick",p.cancelfunc)
        end
    end
    if (p.Resize or not p.noResize) then
        local b,x,y=self:FrameAddResizeButton(f)
    end
    if (p.Help and not p.noHelp) then
        if (p.Helptext) then
            pp("Adding help")
            self:AttachHelp(f,p.Helptext)
        end
    end
    self:MakeMovable(p.name,p.noTip)
    if (show) then
        f:Show()
    end
    return f
end
function lib:GetXY(f,b)
    local br=b:GetRight() or -1
    local fl=f:GetLeft() or -1
    local bb=b:GetBottom() or -1
    local ft=f:GetTop() or -1
    local x=br-fl
    local y=bb-ft
    return x,y
end
function lib:FrameAddXButton(f)
    local fname=f:GetName() .. "XButton"
    local b=self:Create("Button",fname,f,"UIPanelCloseButton")
    b:SetHeight(32)
    b:SetWidth(32)
    b:SetPoint("TOPRIGHT",f,"TOPRIGHT",0,0)
    b:Show()
    return b
end
function lib:FrameResetCastButtonList(f)
    f.buttonlist=0
end
function lib:FrameAddCastButtonList(f,ord,spelltableorname,target,rank,tipo,mouse)
    ord=ord or 0
    f.buttonlist=f.buttonlist or 0
    local spelltable=spelltableorname
    if (type(spelltableorname)=="string") then
        spelltable={spellname=spelltableorname,spelltarget=target,spellrank=rank,spelltype=tipo,spellmouse=mouse}
    end
    local b=self:FrameAddCastButton(f,f.buttonlist)
    f.buttonlist=f.buttonlist +1
    b:SetPoint("TOPLEFT",f,"BOTTOMLEFT",36*ord,0)
    b:SetScale(0.5)
    b:SetSpell(spelltable)
    b:Show()
    return b
end

local AddCastButton_SetSpell
function AddCastButton_SetSpell(this,t)
    if (type(t) == "string") then
        t={spellname=t,spelltype="spell"}
    end
    t=t or {}
    this.spelltarget=t.spelltarget or "target"
    this.spellmouse=t.spellmouse or ""
    this.spelltype=t.spelltype
    this.spellname=t.spellname or ""
    this.spellrank=t.spellrank or ""
    if (type=="macro") then
        this.spellrank=""
        this.spellmouse=""
    end
    local rank=this.spellrank
    if (rank ~= "") then
        rank="("..this.spellrank..")"
    end
    local nome=this:GetName()
    local icon=getglobal(nome.. "Icon")
    local n=getglobal(nome .. "Name")
    this.icon=icon
    if (icon) then
        local texture
        if (this.spelltype == "macro") then
            _,texture=GetMacroInfo(this.spellname)
        else
            texture=GetSpellTexture(this.spellname)
            if (not texture) then
                texture=GetSpellTexture(this.spellname .. rank)
            end
        end
        texture = texture or "Interface\\Icons\\INV_Misc_QuestionMark"
        icon:SetTexture(texture)
    else
        --ACE:Print("%sIcon not found",nome)
    end
    this.tooltipText=this.spellname .. rank
    if (t.noCast) then
        return
    end
    if (this.target) then
        self:SetAttribute("unit",this.target)
    end
    if (not this.spelltype) then
        return
    end
    this:SetAttribute("type"..this.spellmouse,this.spelltype)
    --ACE:Print("type"..this.spellmouse,this.spelltype)
    this:SetAttribute(this.spelltype .. this.spellmouse,this.spellname .. rank )
    --ACE:Print(this.spelltype .. this.spellmouse,this.spellname ..  this.spellrank)
end
local UpdateCooldown
function UpdateCooldown(spellbutton)
    local type=spellbutton.spelltype
    if (not type) then return end
    if (type == "spell") then
        local c=spellbutton.cooldown
        if (c) then
            local start,duration,enable=GetSpellCooldown(spellbutton.spellname)
            if (start and duration) then
                CooldownFrame_SetTimer(c,start,duration,enable)
            end
        end
    end
end
local Checkrange
function CheckRange(this)
    if (this.spellname) then
        if (SpellHasRange(this.spellname)) then
            if (IsSpellInRange(this.spellname,this.unit) == 0) then
				this.icon:SetVertexColor(1.0, 0.1, 0.1);
			elseif (IsUsableSpell(this.spellname)) then
    			this.icon:SetVertexColor(1.0, 1.0, 1.0);
            else
                this.icon:SetVertexColor(0.1,0.1,1.0)
			end
        end
    end
end
function lib:FrameAddCastButton(f,name,x,y,spell)
    name=name or ""
    local fname=f:GetName() .. "Cast" ..name
    b=self:Create("CheckButton",fname,f,"SecureActionButtonTemplate,ActionButtonTemplate")
    b:RegisterForClicks("AnyUp")
    b:SetHeight(32)
    b:SetWidth(32)
    b:SetAttribute("useparent-unit", true);
    b.unit=b:GetAttribute("unit") or f:GetAttribute("unit")
    b.SetSpell=AddCastButton_SetSpell
    b.cooldown=getglobal(fname .. "Cooldown")
    b:SetScript("OnEnter",function() self:Print("entering tooltip") self.TTOpen() end)
    b:SetScript("OnLeave",self.TTClose)
    b:SetScript("PostClick",UpdateCooldown)
    b:SetScript("OnEvent",
        function(this,event)
            if (event == "ACTIONBAR_UPDATE_COOLDOWN") then
                UpdateCooldown(this)
                this:SetChecked(false)
            end
        end
        )
    b.rangetimer=0.5
    b:SetScript("OnUpdate",
        function(this,elapsed)
    		this.rangetimer = this.rangetimer - elapsed;
    		if ( this.rangetimer <= 0 ) then
			     CheckRange(this)
			     this.rangetimer=0.5
            end
		end
		)
    b:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
    if (x and y) then
        b:SetPoint("TOPLEFT",x,y)
    end
    if (spell) then
        b:SetSpell(spell)
    end
    b:Show()
    return b
end
local AddSpellButton_OnReceiveDrag
local AddSpellButton_SetSpell
local AddSpellButton_OnMouseUp
local AddSpellButton_SetText
function AddSpellButton_OnMouseUp(this,button)
    if (not CursorHasMacro() and not CursorHasSpell()) then
        if (button == "RightButton") then
            this:SetSpell(nil)
            return
        end
    end
    AddSpellButton_OnReceiveDrag(this)
end
function AddSpellButton_OnReceiveDrag(this)
    if (not CursorHasMacro() and not CursorHasSpell()) then
        return
    end
    local tipo,slot,book=GetCursorInfo()
    local spellname
    local rank
    if (tipo == "spell") then
        spellname,rank=GetSpellName(slot,book)
    end
    if (tipo == "macro") then
        spellname=GetMacroInfo(slot)
        rank=""
    end
    if (not spellname) then
        self:Debug("You cant drag this one!")
        return
    end
    this.spelltype=tipo
    this.spellrank=rank
    this.spellname=spellname
    this:SetSpell{spelltype=tipo,spellrank=rank,spellname=spellname}
    this:GetName()
    ClearCursor()
end
function AddSpellButton_SetText(this,testo)
    _ = this.Label and this.Label:SetText(testo)
end
function AddSpellButton_SetSpell(this,t)
    t=t or {}
    t.noCast=true
    AddCastButton_SetSpell(this,t)
    pcall(function() this:SetText(t.spellrank or t.spellname) end)
end
function lib:FrameAddSpellButton(f,name,x,y,spell,label)
    name=name or ""
    local fname=f:GetName() .. "Cast" ..name
    local b=self:Create("CheckButton",fname,f,"ActionButtonTemplate")
    b.Type=nil
    b.name="Empty"
    b:RegisterForClicks("RightButtonUp")
    b:SetScript("OnDragStart",nil);
	b:SetScript("OnReceiveDrag",AddSpellButton_OnReceiveDrag)
	b:SetScript("OnMouseUp",AddSpellButton_OnReceiveDrag)
	b:SetScript("OnClick",function(this) AddSpellButton_SetSpell(this,nil) end)
	b:SetScript("OnEnter",function() self:TTOpen() end)
	b:SetScript("OnLeave",self.TTClose)
	b.SetSpell=AddSpellButton_SetSpell
    b:SetHeight(32)
    b:SetWidth(32)
    b:SetPoint("TOPLEFT",x or 0,y or 0)
    if (label) then
        b.Label=self:FrameAddLabel(b,"Text" ,150,34,0)
    	b.SetText=AddSpellButton_SetText
    else
        b.SetText=function() return end
    end
    if (spell) then
        b:SetSpell(spell)
    end
    b:Show()
    return b
end
function lib:FrameAddHelpButton(f)
    local fname=f:GetName() .. "BtHelp"
    local b=self:Create("Button",fname,f,"UIPanelCloseButton")
    b:SetNormalTexture("interface\\buttons\\ui-microbutton-help-up")
    b:SetPushedTexture("interface\\buttons\\ui-microbutton-help-down")
    b:SetHighlightTexture("interface\\buttons\\ui-microbutton-hilight")
    b:SetHeight(32)
    b:SetWidth(16)
    b:SetPoint("TOPLEFT",f,"TOPRIGHT",-5,5)
    b:SetScript("OnClick",nil)
    b:RegisterForClicks(nil)
    b:Show()
    return b
end
function lib:FrameAddResizeButton(f)
    f=self:IsFrame(f)
    local frame=f:GetName()
    local b=self:Create("Button",fname,f,"UIPanelCloseButton")
    b:SetHeight(16)
    b:SetWidth(16)
    b:SetPoint("TOPLEFT",f,"TOPLEFT",0,0)
    b.texture="plus"
    b:SetNormalTexture("interface\\buttons\\ui-plusbutton-up")
    b:SetPushedTexture("interface\\buttons\\ui-plusbutton-down")
    b:SetHighlightTexture("interface\\buttons\\ui-plusbutton-hilight")
    self:TTAdd(b,"Rescale")
    b:SetScript("OnClick",
        function(this,button)
            if (button == "LeftButton") then
                if (this.texture =="plus") then
                    this:SetNormalTexture("interface\\buttons\\ui-minusbutton-up")
                    this:SetPushedTexture("interface\\buttons\\ui-minusbutton-down")
                    this:SetHighlightTexture("interface\\buttons\\ui-minusbutton-hilight")           this.texture="minus"
                else
                    this:SetNormalTexture("interface\\buttons\\ui-plusbutton-up")
                    this:SetPushedTexture("interface\\buttons\\ui-plusbutton-down")
                    this:SetHighlightTexture("interface\\buttons\\ui-plusbutton-hilight")           this.texture="plus"
                end
                self:Rescale(this)
            elseif (button == "RightButton") then
                self:PrintLiteral(f.storage)
            end
        end)
    b:Show()
    return b
end
function lib:Rescale(button)
    local form=button:GetParent()
    local current=math.floor(form:GetScale()*100)
    local min=10
    local max=200
    local fname=form:GetName() .. "SliderResizer"
    local f=getglobal(fname)
    if (not f) then
        f =self:FrameAddSlider(form,"Resizer","",min,max,current)
        f:Hide()
    end
    f:ClearAllPoints()
    f:SetPoint("TOPLEFT",0,50)
    if (f) then
        f.percentage=true
        f.decimals=false
        if (f:IsShown()) then
            f:Hide()
            return
        end
        local func=form:GetScript("OnHide")
        form:SetScript("OnHide",function ()
                                        f:Hide()
                                        pcall(func)
                                    end)


        f:Show()
        f:ClearAllPoints()
        f:SetPoint("BOTTOMLEFT",form,"TOPLEFT", 0, 5)
        f:Show()
        f:SetScript("OnMouseUp",
            function()
                self:Scale(form,this:GetValue()/100,true)
                self:PanelSave(form)
            end)
    end
end
function lib:FrameAddSlider(f,tipo,testo,min,max,current,x,y)
    local fname=f:GetName() .. 'Slider' .. tipo
    min=min or 0
    max=max or 100
    current=current or 50
    s=self:Create("Slider",fname,f,"OptionsSliderTemplate")
    s:SetScript("OnShow",function(this) self:SliderUpdate(this) end)
    s:SetScript("OnValueChanged",function(this) self:SliderUpdate(this) end)
    s:SetMinMaxValues(min,max)
    s:SetValue(current)
    s:SetPoint("TOPLEFT",x or 0,y or 0)
    s:Show()
    return s
end
function lib:SliderUpdate(Slider)
    local frame=Slider:GetParent()
    local f=frame:GetName()
    if (not frame.percentage and not frame.decimals) then
        frame.decimals=1
    end
    if (f) then
    	local vcurrent=Slider:GetValue()
        local formato="%"
        if (frame.decimals) then
            formato=formato.. "." .. frame.decimals .. "f"
            vcurrent=math.floor(vcurrent * 10 ^ frame.decimals)/10 ^ frame.decimals
        else
            formato=formato .. "d"
            vcurrent=math.floor(vcurrent)
        end
        if (frame.percentage) then
            perc="%"
        else
            perc=""
        end
        local sn=Slider:GetName()
    	local vmin,vmax=Slider:GetMinMaxValues()
    	local x=getglobal(sn .. "Low")
    	if (x) then
    		x:SetText(format(formato,vmin) .. perc)
    	end
    	x=getglobal(sn .. "High")
    	if (x) then
    		x:SetText(format(formato,vmax) .. perc)
    	end
    	x =getglobal(sn .. "Text")
    	if (x) then
    		x:SetText(format(formato,vcurrent) .. perc)
    	end
    end
end

function lib:FrameAddUnitButton(f,name,testo,unit,x,y)
    local b=self:FrameAddButton(f,name,testo,x,y,",SecureActionButtonTemplate")
    b:SetAttribute("type1","macro")
    b:SetAttribute("macrotext1","/target " .. unit)
    b.SetUnit=function(this,unit)
        this:SetAttribute("macrotext1","/target " .. unit)
        this:SetText(unit)
        end
    return b
end
function lib:FrameAddButton(f,tipo,testo,x,y,moretemplate)
    moretemplate = moretemplate or ''
    tipo=tipo or 'AMO'
    testo=testo or tipo
    tipo=string.capitalize(tipo)
    local fname=f:GetName() .. "Bt" .. tipo
    local b=self:Create("Button",fname,f,"UIPanelButtonTemplate" .. moretemplate)
    local width=0
    b:SetText(testo)
    width=b:GetTextWidth()
    if (width < 64) then
        width=64
    elseif (width >200) then
        width=200
    end
    b:SetHeight(32)
    b:SetWidth(width)
    fs=nil
    local function closefunc()
        local a=this:GetParent()
        if (a) then
            a:Hide()
        end
    end
    if (tipo=="Close") then
        self:TTAdd(b,"Close this panel")
        b:SetPoint("BOTTOM",f,"BOTTOM",0,-10)
        b:SetScript("OnClick",closefunc)
    elseif (tipo == "Cancel") then
        self:TTAdd(b,"Exit without saving your changes")
        b:SetPoint("BOTTOMLEFT",f,"BOTTOMLEFT",30,30)
        b:SetScript("OnClick",closefunc)
    elseif (tipo=="Save") then
        self:TTAdd(b,"Save your changes and exit")
        b:SetPoint("BOTTOMRIGHT",f,"BOTTOMRIGHT",-30,30)
        b:SetScript("OnClick",closefunc)
    else
        local x = x or 0
        local y = y or 0
        b:SetPoint("TOPLEFT",x,y)
    end
    b:Show()
    return b,self:GetXY(f,b)
end
function lib:FrameMove(f,x,y)
    if (type(f)=="string") then
        f=getglobal(f)
    end
    if (f) then
        local a1,p,a2,ox,oy=f:GetPoint()
        f:SetPoint(a1,p,a2,x or ox,y or oy)
    end
end
function lib:FrameAddCloseButton(f)
    return self:FrameAddButton(f,"Close")
end
function lib:FrameAddSaveButton(f)
    return self:FrameAddButton(f,"Save")
end
function lib:FrameAddCancelButton(f)
    return self:FrameAddButton(f,"Cancel")
end
function lib:TTAdd(frame,message)
    if (not message) then
        return
    end
    if (type(frame) == "string") then
        frame=getglobal(frame)
    end
    if (type(frame)=="table") then
        frame:SetScript("OnEnter",
                        function()
                            self:TTOpen()
                        end)
        frame:SetScript("OnLeave",
                        function()
                        self:TTClose()
                        end)
        frame.Tooltip=tostring(message)
    else
        self:Errorf("AMO_AddTooltip: unable to create tooltip %s for frame %s",message,frame)
    end
end

function lib:FrameAddLabel(f,nome,testo,x,y)
    self:Debug(f,nome,testo,x,y)
    local width=f:GetWidth()
    if (type(testo) == "number") then
        width=testo
        testo=""
    else
        testo =testo or ''
    end
    local fname=f:GetName() .. nome
    local frame=self:Create("Frame",fname,f)
    frame:SetPoint("TOPLEFT",x or 0,y or 0)
    frame:SetWidth(width)
    frame:SetHeight(32)
    if (not frame.Label) then
        frame.Label=f:CreateFontString(fname .. "Label","OVERLAY","GameFontNormal")
    end
    frame.Label:SetText(testo)
    frame.Label:SetAllPoints(frame)
    frame.SetText=function(this,testo) this.Label:SetText(testo) end
    frame.GetText=function(this) return this.Label:GetText() end
    return frame
end

function lib:FrameAddTitle(f,nome,testo,x,y)
    self:Debug(f,nome,testo,x,y)
    x=0
    y=y or 0
    local frame=self:FrameAddLabel(f,nome,testo,x,y)
    frame:SetPoint("TOPRIGHT",x,y) -- So it's resized automaigcally
    self:Debug(frame:GetPoint())
    frame.Label:SetJustifyH("CENTER")
    return frame
end

function lib:FrameAddCLabel(f,nome,testo,x,y)
    self:Debug(f,nome,testo,x,y)
    local frame=self:FrameAddLabel(f,nome,testo,x,y)
    self:Debug(frame:GetPoint())
    frame:SetWidth(frame.Label:GetStringWidth())
    frame.Label:SetJustifyH("CENTER")
    frame.Label:CanNonSpaceWrap(false)
    return frame
end

function lib:FrameAddLLabel(f,nome,testo,x,y)
    self:Debug(f,nome,testo,x,y)
    local frame=self:FrameAddLabel(f,nome,testo,x,y)
    self:Debug(frame:GetPoint())
    frame:SetWidth(frame.Label:GetStringWidth())
    frame.Label:SetJustifyH("LEFT")
    frame.Label:CanNonSpaceWrap(false)
    return frame
end


function lib:FrameAddRLabel(f,nome,testo,x,y)
    self:Debug(f,nome,testo,x,y)
    local frame=self:FrameAddLabel(f,nome,testo,x,y)
    self:Debug(frame:GetPoint())
    frame:SetWidth(frame.Label:GetStringWidth())
    frame.Label:CanNonSpaceWrap(true)
    frame.Label:SetJustifyH("RIGHT")
    return frame
end


function lib:FrameAddIcon(f,tipo,texture,x,y)
    local b=f:CreateTexture(f:GetName()  .. tipo .. "Icon")
    x=x or 0
    y=y or 0
    b:SetPoint("TOPLEFT",x,y)
    b:SetWidth(16)
    b:SetHeight(16)
    b:SetTexture(texture)
    --b:SetHeight(32)
    --b:SetWidth(32)
    return b
end
function lib:FrameAddPanel(f,nome,attach,x,y)
    self:Debug(f,nome,attach,x,y)
    local width=f:GetWidth()
    x = x or 0
    y = y or 0
    attach=attach or 'none'
    attach=strupper(attach)
    local fname=f:GetName() .. nome
    local frame=self:Create("Frame",fname,f)
    self:SetBackdrop(frame,f.storage.Backdrop)
    self:Print("lib:CreatePanel -> ","child")
    if (attach=='LEFT') then
        frame:SetPoint("TOPRIGHT",f,"TOPLEFT",0,0)
        frame:SetPoint("BOTTOMRIGHT",f,"BOTTOMLEFT",0,0)
    elseif (attach=='TOP') then
        frame:SetPoint("BOTTOMLEFT",f,"TOPLEFT",0,0)
        frame:SetPoint("BOTTOMRIGHT",f,"TOPRIGHT",0,0)
    elseif (attach=='RIGHT') then
        frame:SetPoint("TOPLEFT",f,"TOPRIGHT",0,0)
        frame:SetPoint("BOTTOMLEFT",f,"BOTTOMRIGHT",0,0)
    elseif (attach=='BOTTOM') then
        frame:SetPoint("TOPLEFT",f,"BOTTOMLEFT",0,0)
        frame:SetPoint("TOPRIGHT",f,"BOTTOMRIGHT",0,0)
    else
        f:SetPoint("TOPLEFT",x,y)
    end
    frame:SetWidth(width)
    frame:SetHeight(100)
    return frame
end
function lib:TTClose()
	GameTooltip:FadeOut()
end

function lib:TTFade()
	GameTooltip:FadeOut()
end
function lib:TTFormat(lmessage,rmessage)
    lmessage=lmessage or ''
    rmessage=rmessage or ''
    self:TTOpen(
        AMO_GREEN .. lmessage .. AMO_GREEN .. rmessage .. AMO_ENDCOLOR
    )
end

function lib:TTOpen(message)
    if (this:GetAlpha() < 0.1) then
        return
    end
    local ex=0
    if (type(message)~="string") then
        if (type(this.tooltipFunction) == "function") then
            message=this:tooltipFunction(message)
            ex=1
        elseif (type(this.tooltipText)=="string") then
            message=this.tooltipText
            ex=2
        elseif (type(this.Tooltip)=="string") then
            message=this.Tooltip
            ex=3
        else
            message=nil
            ex=4
        end
    end
    if (not message) then
        return
    end
    if (tostring(message) == "nil") then
        return
    end
	if ( message ) then
		GameTooltip:SetOwner(this, "ANCHOR_CURSOR");
		GameTooltip:SetText(message, nil, nil, nil, nil, 1);
	end
	if ( this.tooltipRequirement ) then
		GameTooltip:FrameAddLine(this.tooltipRequirement, "", 1.0, 1.0, 1.0);
	end
	GameTooltip:Show();

end
lib.PoiiOffsets={
MiNN=0, -- Neutral mine
MiHH=1,   -- Horde mine
MiAA=2,   -- Ally mine
GyAN=3,   -- Ally contested Graveyard
HoNN=4,   -- Neutral Home
ToNN=5,   -- Neutral Tower
ObNN=6,   -- Objective
GyNN=7,   -- Neutral Graveyard
ToAN=8,   -- Ally contested Tower
ToHH=9,   -- Horde Tower
ToAA=10,  -- Ally Tower
ToHN=11,  -- Horde Contested Tower
GyHH=12,  -- Horde Graveyard
GyHN=13,  -- Horde Contested Graveyard
GyAA=14,  -- Ally Graveyard
GmNN=16,  --Golden Mine
GmAN=17,
GmAA=18,
GmHN=19,
GmHH=20,
LmNN=21,  --Lumber Mill
LmAN=22,
LmAA=23,
LmHN=24,
LmHH=25,
BsNN=26,  --BlackSmith
BsAN=27,
BsAA=28,
BsHN=29,
BsHH=30,
FaAA=31,  --Farm
FaAN=32,
FaAA=33,
FaHN=34,
FaHH=35,
StAA=36,  --Stables
StAN=37,
StAA=38,
StHN=39,
StHH=40,
SkNN=41,  -- Skull
SkXX=42,  --Dunno...
FlAA=43,
FlHH=44,
FlNN=45,
BaAA=46,
BaAN=47,
BaHH=48,
BaHA=49,
INVI=55
}
function lib:SetPoiiOffset(f,typ)
if (f and typ) then
    local splitter=8
    local row,col=self:Split(typ,splitter) -- 8 icons each row
    f:SetTexCoord(1/splitter*col,1/splitter*(col+1),1/splitter*row,1/splitter*(row+1))
end
end
----------------------------------------
function lib:GetTooltip()
    if (not self.LocalTip) then
        self.LocalTip=self:Create("GameTooltip","AMO_Tip",UIParent,"GameTooltipTemplate")
        self.LocalTip:SetOwner(UIParent, "ANCHOR_NONE")
    end
    return self.LocalTip
end
function lib:CreateTargetPanel(nome,show,unit)
    local f=self:Create("Panel",nome,UIParent)
    f:SetHeight(37)
    f:SetWidth(94)
    self:FrameAddXButton(f)
    f.button=self:FrameAddUnitButton(f,"target",unit,unit,0,-5)
    f.SetUnit=(function(this,unit)
        this.button:SetUnit(unit)
        end)
end
function lib:CreateCastPanel(nome,show,spell)
    local f=self:Create("Panel",nome,UIParent)
    f:SetHeight(80)
    f:SetWidth(94)
    self:FrameAddXButton(f)
    local b=self:FrameAddCastButton(f,"")
    f.SpellButton=b
    b:SetScript("PostClick",function(this)
          this:GetParent():Hide()
          this:GetParent():SetAlpha(0)
    end)
    if (spell) then
        b:SetSpell(t)
    end
    f.SetSpell=function(this,t) this.SpellButton:SetSpell(t) end
    if (show) then
        f:Show()
    end
    return f
end
function lib:Create(tipo,nome,frameobject,template)
    local frameobject=frameobject or UIParent
    local isPanel
    if (tipo=="Panel") then
        isPanel=true
        tipo="Frame"
    end
    local f=getglobal(nome)
    if (not f) then
        r,f=pcall(CreateFrame,tipo,nome,frameobject,template)
        if (not r) then
            self:PrintLiteral(tipo,nome,frameobject,template)
            self:Error("Error AMO_Create: " .. f .. "-" ..tostring(nome))
            return nil
        end
    end
    if (isPanel) then
        f.storage=self.db.profile.forms[nome]
    end
    f.objectname=nome
    return f
end
---------- Event System --------------------------------
function lib:Every(func,elapsed,every,key)
    if (not func) then
        return
    end
    if (not self.Timers) then
        self.Timers={}
    end
    key = key or tostring(func)
    if (not self.Timers[key]) then
        self.Timers[key]=0
    end
    self.Timers[key]=self.Timers[key] + elapsed
    while (self.Timers[key] > every) do
        if (type(func) == "function") then
            func()
        else
            self:Print("Called with wrong func")
        end
        self.Timers[key]=self.Timers[key] - every
    end
end

function lib:EvtPLAYER_REGEN_ENABLED()
end
function lib:EvtPLAYER_REGEN_DISABLED()
end
function lib:EvtACTIONBAR_SLOT_CHANGED(numslot)
    self.dirtybar=numslot
end
function lib:EvtUPDATE_BINDINGS()
    self:GetAllBinds()
end
function lib:SpellCache(key)
    if (not lib.spellcache) then
        --Cache Creation
        lib.spellcache={}
        local i = 1
        while true do
           local spell, rank = GetSpellName(i, BOOKTYPE_SPELL)
           if (not spell) then
              break
           end
           lib.spellcache[spell]=i
           if (rank) then
              spell = spell.." ("..rank..")"
           end
           lib.spellcache[spell]=i
           i = i + 1
        end
    end
    if (key) then
        return lib.spellcache[key]
    end
end
---------------- Deprecated functions
AMO_AddTooltip=AMO_TTAdd
--- For Version 2.0.1 20001
function lib:PushBind(action,key1,key2)
    if (not action) then
        return
    end
    local mod,command=string.split("_",action)
    if (not mod) then
        return
    end
    if (self.Modules[mod]) then
        local modulo=getglobal(mod)
        local func=modulo['Bind']
        if (type(func)=="function") then
            local rc,message=pcall(func,modulo,command,key1,key2)
            if (not rc) then
                self:Error("Errore on action %s (%s)",action,message)
            end
        end
    end
end
function lib:GetAllBinds(show)
    local header=nil
    for i=1,GetNumBindings() do
        action,key1,key2=GetBinding(i)
        local prefix=string.split("_",action)
        if (show) then
            self:Print(self:SafeConcat(" ",action,key1,key2))
        elseif (prefix == "HEADER") then
            header = action == "HEADER_AMO_HEADER"
        elseif (header) then
            lib:PushBind(action,key1,key2)
        end
    end
end

function lib:MoneyFormat(amount)
    local c,s,g
    amount=tonumber(amount) or 0
    local signus=AMO_GREEN .. "got "
    if (amount < 0 ) then
        signus=AMO_RED .. "lost " .. AMO_ENDCOLOR
        amount=amount * -1
    end
    c=amount % 100
    amount=floor(amount / 100)
    s=amount % 100
    amount= floor(amount/100)
    g=amount
    g=AMO_GOLD .. g .. "g."
    s=AMO_SILVER .. s .. "s."
    c=AMO_COPPER .. c .. "c" .. AMO_ENDCOLOR
    return signus.. g..s..c
end
function lib:Split(id,rowlen)
    local row=math.floor(id/rowlen);
    local col=id-(row*rowlen)
    return row,col
end
function lib:TimeToStr(time) -- Converts time data to a string format
	local s, m , h;
	if (not time) then
        return ("0:00")
    end
	s = math.floor(mod(time, 60));
	m = math.floor(time/ 60);
	if (m > 59) then
        h=math.floor(m/60)
        m=math.floor(mod(m,60))
    end
    if (h) then
	   if ( m < 10 ) then
            m = "0" .. m;
       end
       h=h .. ":"
    else
        h=''
    end
	if ( s < 10 ) then
        s = "0" .. s;
    end

	return m .. ":" .. s;
end

function lib:SetBackdrop(f,tipo)
    if (type(f) == "string") then
        f=getglobal(f)
    end
    if (not f) then
        return
    end
    if (type(tipo) == "table") then
        pcall(f.SetBackdrop,f,tipo)
    elseif (not tipo or not Backdrops[tipo]) then
        f:SetBackdrop(nil)
    else
        if (f.storage) then
            f.storage.Backdrop=tipo
        end
        pcall(f.SetBackdrop,f,Backdrops[tipo])
    end
    return f:GetBackdrop()
end
function lib:iCmdButton(arg1)
    arg1=arg1 or 'Heal'
    b=self:CreateCastPanel("amo_b",true,arg1)
end
function lib:iCmdTarget(arg1)
    arg1=arg1 or 'player'
    b=self:CreateTargetPanel("amo_t",true,arg1)
end
function lib:iCmdPopup()
	local anchor = CreateFrame("CheckButton", "AnchorEnterTest", UIParent, "SecureAnchorEnterTemplate")
	anchor:SetPoint("TOPLEFT", UIParent, "CENTER")
	anchor:SetAttribute("*childraise-OnEnter", true)
	anchor:SetAttribute("*childstate-OnEnter", "enter");
	anchor:SetAttribute("*childstate-OnLeave", "leave");
	local hdr = CreateFrame("Frame", "AnchorEnterTest_hdr", anchor, "SecureStateHeaderTemplate")
	hdr:SetPoint("CENTER")
	hdr:SetAttribute("statemap-anchor-enter", "1")
	hdr:SetAttribute("statemap-anchor-leave", ";") -- a nonempty statemap.  to work with the delay
	hdr:SetAttribute("delaystatemap-anchor-leave", "1:0");
	hdr:SetAttribute("delaytimemap-anchor-leave",  "1:1");
	hdr:SetAttribute("delayhovermap-anchor-leave", "1:true");
	anchor:SetAttribute("anchorchild", hdr)
	for i = 1,6 do
		local btn = CreateFrame("CheckButton", "AnchorEnterButton"..i, hdr, "ActionBarButtonTemplate")
		btn:SetAttribute("ofspoint", "*:LEFT");
		btn:SetAttribute("ofsrelpoint", "*:RIGHT");
		btn:SetAttribute("ofsx", 40 * i);
		btn:SetAttribute("ofsy", 0);
		btn:SetAttribute("*type*", "action")
		btn:SetAttribute("*action*", 114+i)
		btn:SetAttribute("hidestates", 0)
		hdr:SetAttribute("addchild", btn)
	end
    anchor:SetHeight(AnchorEnterButton1:GetHeight())
	anchor:SetWidth(AnchorEnterButton1:GetWidth() * 6)
end

function lib:Popup(msg,timeout)
msg=msg or "Something strange happened"
       StaticPopupDialogs["AMO_POPUP"] = {
            text = TEXT(msg),
            button1 = TEXT("OK"),
            showAlert = 1,
            timeout = timeout or 60,
            exclusive = 1,
            whileDead = 1,
            interruptCinematic = 1
        };
        StaticPopup_Show("AMO_POPUP");
end
function lib:Cmd(cmdline)
    local cmd,subcmd,param,fullarg=self:Parse(cmdline)
    cmd=string.capitalize(cmd or "help")
    local funcname=string.capitalize(cmd)
    local func=self["Cmd" .. funcname]
    if (not func) then
        func=self["iCmd" .. funcname]
    end
    if (type(func)=="function") then
        self:Debug("Eseguo")
        func(self,subcmd,param,fullarg)
    else
        self:ShowHelp()
    end
end
function lib:CmdConfig()
    local fname=self.fullname.."Config"
    local p=self.ID
    AceLibrary("Waterfall-1.0"):Open(self.ID)
end
lib.CmdGui=lib.CmdConfig -- alias
function lib:CmdReset()
    self:ResetDB()
    self.db.profile.settings={}
    for k,t in kpairs(self.Config) do
        self:Add(t)
    end
end
function lib:CmdHelp()
    self:Debug("Helppage: %s",self.helppage)
    self:ShowHelp(self.helppage)
end
function lib:CmdStatus()
    self:Print("Variables status for %s",self.fullname)
    local db=self.db.profile.settings
    for i,v in kpairs(db) do
        self:Print("%s: %s (%s)",i,v.value,v.type)
    end
end
--[[
function lib:CmdProfile()
    local s,l=self:GetProfile()
    self:Printf("%s (%s)",s,l)
end
--]]
function lib:iCmdWatcher()
    local f=self:Create("Frame","AMO_FrameWatcher",UIParent)
    f:SetHeight(20)
    f:SetWidth(250)
    f:SetPoint("TOP")
    local l=self:FrameAddTitle(f,"Frame","",10,-30)
    f.label=l
    self:SetBackdrop(f,"tooltip")
    f:SetScript("OnUpdate",
        function(elapsed)
            local frame=GetMouseFocus()
            if (frame ~= this.oldframe and frame) then
                this.oldframe=frame
                this.label:SetText(frame:GetName())
            end
        end)
end
function lib:ScanBuff(unit)
    return lib:ScanBuffDebuff(unit)
end
function lib:ScanDebuff(unit)
    return lib:ScanBuffDebuff(unit,true)
end
function lib:ScanBuffDebuff(unit,debuff)
    local f=UnitBuff
    if (debuff) then
        f=UnitDebuff
    end
    if (not unit) then
        unit="player"
    end
    local buffs={}
    for i=1,99 do
        local buffName,buffRank,buffTexture,buffApplications,debuffType=f(unit,i)
        if (buffName) then
            local tx=string.gsub(buffTexture,"Interface\\Icons\\","")
            buffs[tx]={name=buffName,rank=buffRank,texture=buffTexture,applications=buffApplications,type=debuffType}
            -- type= Magic Disease Poison Curse or nothing
        else
            break
        end
    end
    return buffs
end

function lib:CheckSelf()
    self:Print("Called from ",self.ID)
end
-- Help Syste,

function lib:LoadHelp(testo)
    if (testo) then
        self:Print(self.ID .. "Old use of loadhelp")
        return
    end
    ACH_PAGENUM=ACH_PAGENUM+1
    ACH_HELP_TITLES[ACH_PAGENUM]=self.fullname .. " " .. self.version
    self:HF_Paragraph("Alar's site")
    self:HF_Pre("http://wow.aspide.it")
    ACH_HELP_PAGES[ACH_PAGENUM]=self.helptext
    ACH_CONFIG[ACH_PAGENUM]=self.ID
    self.helppage=ACH_PAGENUM
    self.helptext=""
end


function lib:HF_Title(text)
    text=text or ''
    self.helptext=self.helptext..AMO_YELLOW .. text
end
function lib:HF_Cmd(cmd,parameters,text,tooltip)
    if (type(cmd) ~= "string") then
        return
    end
    if (type(parameters)=="string") then
        parameters=text
        text=tooltip
    end
    text=text or cmd
    tooltip=tooltip or text
    self.Cmds[cmd]=text
    if (string.sub(cmd,1,1) ~= "/" ) then
        cmd="/" .. strlower(self.ID).. " " .. cmd
    end
    self:HF_Command(cmd,text)
end
function lib:HF_Command(cmd,text)
    text=text or ''
    self.helptext=self.helptext..format("%s%s|r : %s\n",AMO_ORANGE,cmd,text)
end
function lib:HF_Paragraph(text)
    text=text or ''
    self.helptext=self.helptext..format("\n%s%s|r\n",AMO_GREEN,text)
end
function lib:HF_Pre(testo)
    self.helptext=self.helptext..tostring(testo)
end
function lib:HF_Commands()
    modulo=strlower(self.ID)
    self:HF_Paragraph("General commands")
    self:HF_Command("/"..modulo.." help","Opens this help page")
    self:HF_Command("/"..modulo.." gui","Opens GUI (if available)")
    self:HF_Command("/"..modulo.." status","Shows current settings")
end
function lib:HF_Enabled(n,v,d)
    self.helptext=self.helptext..lib:Format("Module %s%s|r %s (%s): "..AMO_GREEN.."Enabled|r\n",AMO_ORANGE,n,v,d)
end
function lib:HF_Disabled(modulo)
    self.helptext=self.helptext..lib:Format("Module $s%s|r %s (%s): "..AMO_RED.."Disabled|r\n",AMO_ORANGE,n,v,d)
end
function lib:HF_Color(color,text)
    self.helptext=self.helptext..lib:Format("%s%s|r",getglobal("AMO_" .. strupper(color),text))
end
-- Events
function lib:RemoveEvent(eventi)
    if (not eventi) then
        return
    end
    if (type(eventi)=="string") then
        eventi={eventi}
    end
    for i,evento in ipairs(eventi) do
        self.Events[evento]=nil
    end
end
function lib:RemoveBucketEvent(eventi)
    if (not eventi) then
        return
    end
    if (type(eventi)=="string") then
        eventi={eventi}
    end
    for i,evento in ipairs(eventi) do
        self.BucketEvents[evento]=nil
    end
end
function lib:AddEvent(eventi,delayed)
    if (not eventi) then
        return
    end
    if (type(eventi)=="string") then
        eventi={eventi}
    end
    delayed = delayed or 0
    for evento,i in pairs(eventi) do
        self.Events[evento]=delayed
    end
end

function lib:AddBucketEvent(eventi,delayed)
    if (not eventi) then
        return
    end
    if (type(eventi)=="string") then
        eventi={eventi}
    end
    delayed = delayed or 0
    for i,evento in ipairs(eventi) do
        self.BucketEvents[evento]=delayed
    end
end

  -----------------------------------------------------------
  -- function lib:GetTime()
  --
  -- Return game time as (h,m,s) where s has 3 decimals of
  -- precision (though it's only likely to be precise down
  -- to ~20th of seconds since we're dependent on frame
  -- refreshrate).
  --
  -- During the first minute of play, the seconds will
  -- consistenly be "00", since we haven't observed any
  -- minute changes yet.
  --
  --

function lib:TimeGet()
  	if(self.LastMinuteTimer == nil) then
  		local h,m = GetGameTime();
  		return tonumber(h),tonumber(m),0;
  	end
  	local s = GetTime() - self.LastMinuteTimer;
  	if(s>59.999) then
  		s=59.999;
  	end
  	return tonumber(self.LastGameHour), tonumber(self.LastGameMinute), tonumber(s);
  end
  -----------------------------------------------------------
  -- function AMO_GameTime:TimeStamp()
  --
  -- Return game time as (h,m,s) where s has 3 decimals of
  -- precision (though it's only likely to be precise down
  -- to ~20th of seconds since we're dependent on frame
  -- refreshrate).
  --
  -- During the first minute of play, the seconds will
  -- consistenly be "00", since we haven't observed any
  -- minute changes yet.
  --
  --

function lib:TimeTimeStamp()
    local h,m,s=self:Get()
    return self:Format("%02d:%02d:%02d:%02d",self.Day,h,m,s)
end


  -----------------------------------------------------------
  -- function AMO_GameTime:OnUpdate()
  --
  -- Called by: Private frame <OnUpdate> handler
  --
  -- Construct high precision server time by polling for
  -- server minute changes and remembering GetTime() when it
  -- last did
  --

function lib:TimeOnUpdate()
  	local h,m = GetGameTime();
  	if(self.LastGameMinute == nil) then
  		self.LastGameHour = h;
  		self.LastGameMinute = m;
  		return;
  	end
  	if(self.LastGameMinute == m) then
  		return;
  	end
  	if (self.LastGameHour>h) then
        self.Day=self.Day+1
    end
  	self.LastGameHour = h;
  	self.LastGameMinute = m;
  	self.LastMinuteTimer = GetTime();
end

  -----------------------------------------------------------
  -- function AMO_GameTime:Initialize()
  --
  -- Create frame to pulse OnUpdate() for us
  --

function lib:TimeInitialize()
    if (self.Frame) then
        return
    end
    self.Day=0
  	self.Frame = CreateFrame("Frame");
  	self.LastMinuteTimer = GetTime();
  	self.Frame:SetScript("OnUpdate", function() self:TimeOnUpdate(); end);
end

function lib:ZapSpellCache()
    lib.spellcache={}
end

function lib:LibVersion()
    self:Print("%s.%s",MAJOR_VERSION,MINOR_VERSION)
end

local function str2num(versione)
    versione =versione or 0
    if (type(versione) == "string") then
        local a,b=versione:match("(%d*).(%d*)")
        a=a or 0
        b=b or 0
        return a*1000+b
    else
        return versione
    end
end
function lib:UpdateVersion()
    self.db.char.version=str2num(self.version)
end
function lib:IsNewVersion()
    return str2num(self.version) > str2num(self.db.char.version)
end
function lib:VersionIsAtLeast(compareto)
    return str2num(compareto) >= str2num(self.version)
end
-- Config Section
    -- Amo Config System
function lib:Vars()
    return next,self.db.profile.settings,nil
end
function lib:DumpVars(key)
    key=key or "<all>"
    key=strupper(key)
    local db=self.db.profile.settings
    for k,v in pairs(db) do
        k=strupper(k)
        if (key == "<ALL>" or key==k) then
            if (v.type == "boolean") then
                self:PrintBoolean(k,v.value)
            else
                self:Printf("%s is %s%s",k,AMO_WHITE,v.value)
            end
            if (key ==k) then
                return
            end
        end
    end
end
function lib:Add(t)
    local k=t.flagname or self:Format("%s_%d",self.ID,table.getn(self.List))
    self.Config[k]=t
    table.insert(self.List,k)
    if (t.type == "C") then
        self:AddVar(t)
    elseif (t.type == "E") then
        self:AddVar(t)
    elseif (t.type == "B" and t.flagname) then
        self:AddVar(t)
    elseif (t.type == "S") then
        self:AddVar(t)
    end
end
function lib:AddLabel(label,tooltip,colortable)
    colortable=colortable or AMO_Orange
    self:Add{type="L",label=label,tooltip=tooltip,colortable=colortable}
end
function lib:AddButton(flagname,defaultvalue,func,label,tooltip,position)
    position=position or "flow"
    position=strupper(position)
    centered=position=="CENTER"
    flow=position=="FLOW"
    self:Add{type="B",flagname=flagname,defaultvalue=defaultvalue,label=label,func=func,tooltip=tooltip,flow=flow,centered=centered}
end
function lib:AddSlider(flagname,defaultvalue,min,max,label,tooltip,position)
    self:Add{type="S",flagname=flagname,defaultvalue=defaultvalue,label=label,tooltip=tooltip,min=min,max=max}
end
function lib:AddTitle(label,tooltip)
self:Add{type="L",label=label,centered=true,tooltip=tooltip}
end
function lib:AddCheckbox(flagname,defaultvalue,label,tooltip)
self:Add{type="C",label=label,tooltip=tooltip,flagname=flagname,defaultvalue=defaultvalue}
end
-- compatibility with
--AddToggle("RETRACK",true,"Track cast reminder","Reminds you to recast your current tracking")
function lib:Empty(t)
    local value
    if (t=="boolean") then
        value=false
    elseif (t == "string") then
        value = ""
    elseif (t == "number") then
        value=0
    elseif (t == "table") then
        value={}
    elseif (t == "function") then
        value=function() end
    else
        value=nil
    end
    return value
end

function lib:AddToggle(flagname,defaultvalue,label,tooltip)
self:Add{type="C",label=label,tooltip=tooltip,flagname=flagname,defaultvalue=defaultvalue}
end
function lib:AddEdit(flagname,defaultvalue,label,tooltip)
self:Add{type="E",label=label,tooltip=tooltip,flagname=flagname,defaultvalue=defaultvalue}
end
function lib:GetToggle(flagname)
    local db=self.db.profile.settings
    local v=db[flagname];
    if (not v) then
        return false
    elseif (v.value==self:Empty(v.type)) then
        return false
    else
        return v
    end
end
function lib:ReApply()
    local db=self.db.profile.settings
    if (type(self.ApplyAll)=="function") then
        self:ApplyAll()
    else
        for i,v in self:Vars() do
            self:_Apply(i,v.value,true)
        end
    end
end
function lib:Apply(flagname,value)

end
function lib:_Apply(cmd,value,silent)
    cmd=strupper(cmd)
    local func
    if (type(self["Apply" .. cmd]) == "function") then
        func=function()
            self["Apply" .. cmd](self,value)
            end
    else
        func=function()
            self:Apply(cmd,value)
            end
    end
    func()
    if (InCombatLockdown()) then
        self:Print("Scheduling changes for when you leave combat")
        self:ScheduleLeaveCombatAction(func)
    else
        func()
    end
    if (not silent) then
        if (type(value)=="boolean") then
            self:PrintBoolean(cmd,value)
        else
            self:Print("%s is %s",cmd,value)
        end
    end
end
function lib:SetVar (flagname,value)
    local flag=self.db.profile.settings[flagname]
    if (flag) then
        as=flag.type
        if (as =="string") then
            if (value) then
                flag.value=tostring(value)
            else
                flag.value=""
            end
        elseif (as =="number") then
            flag.value=(tonumber(value) or 0)
        elseif (as == "boolean") then
            if (value) then
                flag.value=true
            else
                flag.value=false
            end
        else
            flag.value=value
        end
        self.db.profile.settings[flagname]=flag
        return self.db.profile.settings[flagname].value
    else
        return nil
    end
end
function lib:ApplyVar(flagname,value)
    local old=self:GetVar(flagname)
    if (old ~= value) then
        -- SetVar can change a variable type, so the subsequent test
        -- is not useless ;-P
        local new=self:SetVar(flagname,value)
        if (new  ~= old) then
            self:_Apply(flagname,new)
        end
    end
end
function lib:CreateString(flagname,value)
    self:AddVar{flagname=flagname,defaultvalue=value,type="E"}
end
function lib:CreateNumber(flagname,value)
    value=tonumber(value) or 0
    self:AddVar{flagname=flagname,defaultvalue=value,type="N"}
end
function lib:CreateBoolean(flagname,value)
    self:AddVar{flagname=flagname,defaultvalue=value,type="B"}
end
function lib:AddVar (t)
    if (type(self.db.profile) ~= "table") then
        self:Error("self.db.profile is not a table")
        return
    end
    --self:PrintLiteral(self.db.profile.settings[t.flagname])
    local flag=self.db.profile.settings[t.flagname]
    if (not flag) then
        flag={}
        if (t.type=="E") then --Editbox
            flag.value=t.defaultvalue or ""
            flag.type="string"
        elseif (t.type == "C" or t.type=="B") then -- Checkbox or Button
            flag.value=t.defaultvalue
            flag.type="boolean"
        elseif (t.type == "N" or t.type== "S") then -- Slider or number
            flag.value=(tonumber(t.defaultvalue) or 0)
            flag.type="number"
        end
        if (flag.type) then
            self.db.profile.settings[t.flagname]={}
            self.db.profile.settings[t.flagname].type=flag.type
            self.db.profile.settings[t.flagname].value=flag.value
        end
    end
end
function lib:IsVar(flagname)
    local db=self.db.profile.settings
    return db[flagname]
end
function lib:GetNumber(flagname)
    return self:GetVar(flagname,"number") or 0
end
function lib:GetString(flagname)
    return self:GetVar(flagname,"string") or ""
end
function lib:GetRawVar(flagname)
    return self:GetVar(flagname,"raw") or nil
end

function lib:GetVar(flagname,as)
    local db=self.db.profile.settings
    if (db[flagname]) then
        as=as or db[flagname].type
        if (as=="raw") then
            return db[flagname]
        elseif (as =="string") then
            if (not db[flagname].value) then
                return ""
            else
                return tostring(db[flagname].value)
            end
        elseif (as =="number") then
            return tonumber(db[flagname].value) or 0
        else
            if (db[flagname].value) then
                return true
            else
                return false
            end
        end
    else
        return self:Empty(as)
    end
end
function lib:ToggleVar(flagname)
    local db=self.db.profile.settings
    if (db[flagname]) then
        if (db[flagname].type=="boolean") then
            db[flagname].value= not db[flagname].value
        end
    end
end

--[[
Struttura tabella configurazione=
Array di elementi di configurazione
Ogni elemento ha questa struttura
{
type="C" | "L" | "E" | | "S" | "B",
label="Etichetta"
tooltip="Testo tooltip"
flagname="Nomevariabile"
centered=true|false
}
--]]
function lib:DumpDb()
    for k,v in pairs(self.db.profile.settings) do
        self:PrintLiteral(k,v)
    end
end

function lib:AddCmdA(cmd,t,desc,usage)
    if (type(t) == "table") then
        t.standalone=true
    else
        t={set=t,desc=desc,usage=usage,standalone=true}
    end
    self:AddCmd(cmd,t)
end
function lib:AddCmd(cmd,t,desc,usage,standalone)
    if (type(t) == "string") then
        t={set=t,desc=desc,usage=usage,standalone=standalone}
    end
    t.type=t.type or "text"
    t.name=t.name or cmd
    t.desc=t.desc or "Self explicatory"
    t.get=t.get or false
    t.input=t.input or false
    t.usage=t.usage or "message"
    if (t.standalone) then
        self:RegisterChatCommand({"/" .. cmd},t)
        self:HF_Command("/" .. cmd .. " " .. t.usage ,t.desc)
    else
        self.Commands.args[cmd]=t
        self:HF_Command("/" .. strlower(self.ID) .. " " .. cmd .. " " .. t.usage ,t.desc)
    end
end
-- Start
function lib:Evt(...) --placeholder
   local event = AceLibrary("AceEvent-2.0").currentEvent
end
function lib:OnEvent(event,...)
    local func=self["Evt" .. event]
    if (type(func)=="function") then
        rc,message=pcall(func,self,...)
        if (not rc) then
            self:Error("Evt%s %s",event,message)
        end
    else
        if (type(self.Evt)=="function") then
            rc,message=pcall(self.Evt,self,event,...)
            if (not rc) then
                self:Error("Evt %s", message)
            end
        end
    end
end
function lib:_Evt()
    pcall (self.OnEvent,self,AceLibrary("AceEvent-2.0").currentEvent,arg1,arg2,arg3,arg4,arg5,arg6,arg7,arg8,arg9)
end

function lib:Banner()
    self:Print(AMO_GREEN .. "%s %s ready. Use /%s help for info)",self.fullname,self.version,strlower(self.ID))
end

function lib:SecureShow(f,show)
    local hide=not show
    if (type(f)=="string") then
        f=getglobal(f)
    end
    if (hide) then
        self:ScheduleLeaveCombatAction(function() f:Hide() end)
    else
        self:ScheduleLeaveCombatAction(function() f:Show() end)
    end
end

function lib:Config2Opts()
    if (not self.Opts) then
        local wf=AceLibrary("Waterfall-1.0")
        local titolo= self.fullname .. " " .. self.version
        local Opts={ type="group",
                            desc=titolo,
                            args={},
                            handler=self,
                            name="Configuration Options"
                    }
        local root=Opts
        local gruppo=root.args
    	for order,id in ipairs(self.List) do
            v=self.Config[id]
            local tipo=v.type
            local color=v.colortable or AMO_Yellow
            local ID=string.capitalize(id)
            v.label=v.label or id
            v.label=v.label
            v.tooltip=v.tooltip or v.label
            if (tipo=="L") then -- Label
                gruppo=root.args
                gruppo[ID] = {
                    order=order,
                    name=v.label,
                    type="group",
                    desc=v.tooltip,
                    args={},
                }
                gruppo=gruppo[ID].args
            elseif (tipo == "E") then -- Editbox
                gruppo[ID] = {
                    type="text",
                    passValue=v.flagname,
                    get = "GetString",
                    set = "ApplyVar",
                    usage=v.usage or "<Enter string>",
                }
            elseif (tipo == "B") then -- Button
                gruppo[ID] = {
                    type="execute",
                    func=v.func,
                    passValue=v.flagname,
                }
            elseif (tipo == "C") then -- checkbox
                gruppo[ID] = {
                    type="toggle",
                    get="GetToggle",
                    passValue=v.flagname,
                    set="ApplyVar",
                }
            elseif (tipo == "S") then -- slider
                v.max=v.max or 100
                v.min=v.min or 0
                local int=v.max-v.min
                if (int <0) then
                    int=0.1
                else
                    int=int/100
                end
                gruppo[ID]= {
                    type="range",
                    min=v.min,
                    max=v.max,
                    step=int,
                    bigStep=int * 10,
                    get="GetNumber",
                    passValue=v.flagname,
                    set="ApplyVar",
                }
            end
            if (tipo ~= "L" ) then
                gruppo[ID].order=order
                gruppo[ID].name=v.label
                gruppo[ID].desc=v.tooltip
                gruppo[ID].cmdHidden=true
                if (tipo ~="B") then
                    self.Commands.args[ID]=gruppo[ID];
                end
            end
        end
        self.Opts=Opts
        wf:Register(self.ID,"title",titolo,"aceOptions",self.Opts)
    end
end

function lib:CmdEvents()
    for e in pairs(self.Events) do
        local colore=AMO_ORANGE
        local attivo,funzione=self:IsEventRegistered(e)
        if (attivo) then colore=AMO_GREEN end
        self:Print("Event %s%s by %s%s",colore,e,AMO_ENDCOLOR,funzione)
    end
    for e in pairs(self.BucketEvents) do
        local colore=AMO_ORANGE
        local attivo,funzione=self:IsBucketEventRegistered(e)
        if (attivo) then colore=AMO_GREEN end
        self:Print("BucketEvent %s%s by %s%s",colore,e,AMO_ENDCOLOR,funzione)
    end
end

function lib:AutoEvents(debug)
    for method in pairs(self) do
        _=debug and self:Print(method)
        local s,f,event=string.find(method,"^Evt(.*)")
        if (s) then
            _= debug and self:Print("AutoAdding ",event)
            if (not self.BucketEvents[event]) then
                if (not self.Events[event]) then
                    self.Events[event]=0
                    _= debug and self:Print("Added ",event)
                end
            end
        end
    end
end

local function ae(self,event,td,bucket)
    local method="Evt" .. event
    if (not td) then
        self:UnregisterEvent(event)
        return
    else
        if (type(self[method]) ~= "function") then
            method="_Evt"
        end
    end
    if (bucket) then
        self:RegisterBucketEvent(event,td,method)
    else
        self:RegisterEvent(event,method,td)
    end
    end

function lib:ApplyEvents()
    for event,throttle in pairs(self.Events) do
        ae(self,event,throttle,false)
    end
    for event,delay in pairs(self.BucketEvents) do
        ae(self,event,delay,true)
    end
end
function lib:UnApplyEvents()
    self:UnregisterAllBucketEvents()
    self:UnregisterAllEvents()
end
function lib:DbStart()
    self:RegisterDB(self.dbname,self.dbnamePC)
    self:RegisterDefaults('profile',{
         settings={},
         forms={['*']={Scale=1.0,
                        Point={"CENTER",UIParent,"CENTER",0,0},
                    }
                },
         cache={},
         data={}
         }
    )
    self:RegisterDefaults('account',{
        firstrun=true,
        lastversion=0,
        lastinterface=20200,
        }
    )
end

-- Addon start

function lib:new (o)
    self:TimeInitialize()
    local class=AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0","AceDebug-2.0","AceHook-2.1")
    class:SetDebugging(o.debug)
    class:LevelDebug(1,"New: %s (%s)",o.ID,o.fullname)
    if (not o) then
        class:Error("Nothing to initialize")
    end
    for k,v in pairs(o) do
        class[k]=v
    end
    if (not class.version and class.fullname) then
        class.version=GetAddOnMetadata(class.fullname,"Version"):gsub("[%a%s%$:]","")
    else
        class.version="1.0"
    end
    class.dbnamePC="db"..o.ID
    class.dbname=class.dbnamePC .. "gen"
    class.Config={}
    class.List={}
    class.Cmds=o.Cmds or {}
    class.Cmds.Config="Open config GUI"
    class.Cmds.Gui="Open config GUI"
    class.Cmds.Reset="Reset all data"
    class.Cmds.Help="Show help page"
    class.Cmds.Status="Internal use"
    class.Cmds.Events="Show all hooked events for this addon"
    class.Events=o.Events or {}
    class.BucketEvents=o.BucketEvents or {}
    class.helptext=o.helptext or ''
    class.locale=AceLibrary("AceLocale-2.2"):new("Alar" .. o.ID)
    function class:OnEnable(first)
        if (first) then
            self:DbStart()
            self:_Initialize()
        end
        pp(class.ID .. " OnEnable")
        if (type(self.Enable) == "function") then
            self:Enable(first)
        end
        self:ApplyEvents()

    end
    function class:OnDisable()
        pp(class.ID .. " OnDisable")
        self:UnApplyEvents()
        self:UnhookAll()
    end
    function class:OnInitialize ()
        pp(class.ID .. " OnInitialize")
    end
    function class:_Initialize()
        self.Commands={
            type='group',
            handler = self,
            desc='Commands',
            args= { }
        }

    -- I have been loaded. Called just One Time
        self:LevelDebug(2,"OnInitialize",self.ID)
        --self:PrintLiteral(self.db.profile.settings)

        --- Calling specific initializations
        if (type(self.Initialize) == "function") then
            self:Initialize()
        end
        --------------------------------------
        self:Config2Opts()
        self:LocaleLoad()
        local L=self.L
        local args=self.Commands.args
        for method in pairs(self) do
            local start,_,command=string.find(method,"^Cmd(.*)")
            if (start and type(self[method])=="function" and command~="") then
                local desc=self.Cmds[command] or L[command] or 'No description'
                if (not args[command]) then -- Dont step on things done by Addon private initialization
                    -- I check for a command setup function
                    local cmdargs
                    local rc,cmdargs=pcall(self["Setup" .. command],self,cmdargs)
                    if (rc and cmdargs) then
                        args[command]={
                            type='group',
                            name= command,
                            desc= desc,
                            args= cmdargs
                        }
                    else
                        if (self[method .. 'Args']) then
                            args[command]={
                            type = 'text',
                            name = command,
                            input=1,
                            desc = desc,
                            set = method,
                            get = false,
                            }
                        else
                            args[command]={
                            type = 'execute',
                            name = command,
                            desc = desc,
                            func = method,
                            }
                        end
                        args[command].usage=self[method .. 'Usage'] or ''
                    end
                    if (string.match(command,"^Dbg%w+")) then
                        --args[command].cmdHidden=true
                    end
                end
            end
        end
        self:RegisterChatCommand({"/" .. self.ID,"/" .. self.fullname}, self.Commands)
        self:ReApply()
        self:LoadHelp()
        --self:PrintLiteral(self.db.profile.settings)
    end

    function class:LocaleAdd(locale,key,translation)
        self:LevelDebug(1,locale,key,translation)
        translation=translation or true
        self.Translations=self.Translations or {}
        self.Translations[locale]=self.Translations[locale] or {}
        self.Translations[locale][key]=translation
    end
    function class:LocaleLoad()
        local locale=GetLocale()
        self:LevelDebug(1,"Loading translations",locale)
        if locale ~= "deDE" and locale ~= "koKR" and locale ~= "esES" and locale ~= "frFR" and locale ~= "zhTW" then
        	locale = "enUS"
        end
        self:LocaleAdd(locale,"OK",AMO_GREEN .."Succesfull" .. AMO_ENDCOLOR)
        self:LocaleAdd(locale,"KO",AMO_RED .. "Failed" .. AMO_ENDCOLOR)
        self:LocaleAdd(locale,"WARN",AMO_ORANGE .. "Notice" .. AMO_ENDCOLOR)
        local t=self.Translations or {}
        if (type(t[locale]) ~= "table") then
            locale="enUS"
        end
        if (type(t[locale]) ~= "table") then
            self:Error("No locale at all!!")
            return -- No translations at all!!
        end

        self:LocaleAdd(locale,"LocaleLoaded","Locale " .. locale .. " loaded and enabled")
        self.locale:RegisterTranslations(locale,
            function() return self.Translations[locale] end)
    end
    function class:Register()
        pp("Remapping functions")
        -- Remapping all AMO function to class
        for k,v in pairs(AMO) do
            if (not class[k] or k == "Print" or k=="Error" ) then
                if (type(AMO[k]) == 'function') then
                    -- Doesnt exist in class AND is a function
                    class[k] = v
                end
            end
        end
        self:AutoEvents()
        self:Banner()
    end
    class.L={}
    function class:_L(arg)
        if (self.locale:HasBaseTranslation(arg)) then
            return self.locale:GetTranslation(arg)
        else
            return arg
        end
    end
    local t=getmetatable(L) or {}
    t.__index= function (table,key)
                    local value=class:_L(key)
                    return value
               end
    setmetatable(class.L,t)
    return class
end
function lib:OnEnable(first)
    if (first) then
        self:Print(AMO_GREEN .. "%s.%s ready. Use /%s help for info)",MAJOR_VERSION,MINOR_VERSION,strlower(self.ID))
        self:AddCmd("amo","Cmd","AlarLib","help",true)
        self:RegisterEvent("LEARNED_SPELL_IN_TAB","ZapSpellCache")
        self:RegisterEvent("SPELLS_CHANGED","ZapSpellCache")
    end
end
local a_activate
function a_activate(self,oldLib,oldDeactivate)
    pp("AlarLib activation main" .. MINOR_VERSION,1,0.5,0.5)
    local registered=oldLib
    self.helptext=""
    self.Commands = {
        type='group',
        handler = self,
        desc='Commands',
        args= { }
    }
    if (registered) then
        self.debug=oldLib.debug or false
        pp("AlarLib reactivation" .. MINOR_VERSION,1,0.5,0.5)
        ACH_HELP_TITLES[1]="Alarmod library " .. GetAddOnMetadata("AlarMod","Version") or ""
    else
        pp("AlarLib activation " .. MINOR_VERSION,1,0.5,0.5)
        ---- Help System
        ACH_PAGENUM=1
        ACH_HELP_PAGES={};
        ACH_CONFIG={}
        ACH_HELP_TITLES={}
        ACH_PAGENUM=1
        ACH_HELP_TITLES[1]="About"
        ACH_HELP_PAGES[ACH_PAGENUM] = [[
        |c00FFED1AAuthor:|r Alar of Daggerspine
        |c00FFED1AEmail:|r alar@aspide.it
        |c00FFED1AWebSite:|r http://wow.aspide.it
        |c00FFED1AAceProfile:|r http://www.wowace.com/wiki/Category:Author:Alar
        ]];
    end
end
local a_deactivate
function a_deactivate(self)
    pp("AlarLib deactivation " .. MINOR_VERSION,1,0.5,0.5)
    self=nil
end
local a_external
function a_external(self,major,instance)
    --DEFAULT_CHAT_FRAME:AddMessage("Loaded " .. major,0.5,0.5,0.5)
end
AceLibrary:Register(lib, MAJOR_VERSION, MINOR_VERSION, a_activate, a_deactivate, a_external)
del(lib)
local dummylib=new()
function dummylib:new(o)
    pp( "Instancing AlarLib for " .. o.ID)
    local class=AceLibrary("AlarLib-2.0"):new(o)
    -- I Want override Print,Error,Debug
    return class
end
local ACAVERS=99990
AceLibrary:Register(dummylib,"AlarClass-2.0",ACAVERS)
del(dummylib)
AMO=AceLibrary("AlarLib-2.0")
AMO.dbname="dummy"
AMO.dbname="dummychar"
AMO.db={
    profile={
         settings={},
         forms={},
         cache={},
         data={}
         }
    }
